/* Copyright 1998-2011 Research Foundation State University of New York */

/* This file is part of QuB.                                            */

/* QuB is free software; you can redistribute it and/or modify          */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation, either version 3 of the License, or    */
/* (at your option) any later version.                                  */

/* QuB is distributed in the hope that it will be useful,               */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/* GNU General Public License for more details.                         */

/* You should have received a copy of the GNU General Public License,   */
/* named LICENSE.txt, in the QuB program directory.  If not, see        */
/* <http://www.gnu.org/licenses/>.                                      */

#ifndef QUB_NODEARRAY_H
#define QUB_NODEARRAY_H

class QUB_NodeArray
{
protected:
	QUB_Tree root;
	std::vector<QUB_Tree> nodes;
	int siz;

	void addNode(int i);
	void removeNode(int i);

public:
	QUB_NodeArray()
	: root( QUB_Tree::Create("") ), siz(0)
	{}

	int size()                    { return siz; }
	void resize(int n);

	QUB_Tree operator[] (int ix)  { return nodes[ix+1]; }
	QUB_Tree getRoot()            { return root; }
};


class QUB_NodeStash_iface
{
public:
	virtual void      __cdecl Free() = 0;

	virtual QTR_Impl* __cdecl _getRoot() = 0;

	virtual int       __cdecl request() = 0;
	virtual void      __cdecl replace(int key) = 0;
	virtual QTR_Impl* __cdecl _get(int key) = 0;
};

class QUB_NodeStash : public QUB_NodeStash_iface
{
private:
	QUB_NodeArray nodes;
	set<int>      keys; // that are not signed out

public:
	void      __cdecl Free();

	QTR_Impl* __cdecl _getRoot();

	int       __cdecl request();
	void      __cdecl replace(int key);
	QTR_Impl* __cdecl _get(int key);

	QUB_Tree getRoot();
	QUB_Tree get(int key);
};




#endif
